/*
 * bondgraph.hpp
 *
 *  Created on: Oct 29, 2009
 *      Author: heber
 */

#ifndef BONDGRAPH_HPP_
#define BONDGRAPH_HPP_

using namespace std;

/*********************************************** includes ***********************************/

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iosfwd>


/****************************************** forward declarations *****************************/

class molecule;
class BondedParticle;
class MatrixContainer;

/********************************************** definitions *********************************/

/********************************************** declarations *******************************/


class BondGraph {
public:
  BondGraph(bool IsA);
  ~BondGraph();
  bool LoadBondLengthTable(std::istream &input);
  bool ConstructBondGraph(molecule * const mol);
  double GetBondLength(int firstelement, int secondelement);
  double SetMaxDistanceToMaxOfCovalentRadii(const molecule * const mol);


  /** Returns bond criterion for given pair based on a bond length matrix.
   * This calls either the covalent or the bond matrix criterion.
   * \param *Walker first BondedParticle
   * \param *OtherWalker second BondedParticle
   * \param &MinDistance lower bond bound on return
   * \param &MaxDistance upper bond bound on return
   * \param IsAngstroem whether units are in angstroem or bohr radii
   */
  void getMinMaxDistance(BondedParticle * const Walker, BondedParticle * const OtherWalker, double &MinDistance, double &MaxDistance, bool IsAngstroem);

  double getMaxDistance() const;

private:
  static const double BondThreshold;

  void BondLengthMatrixMinMaxDistance(BondedParticle * const Walker, BondedParticle * const OtherWalker, double &MinDistance, double &MaxDistance, bool IsAngstroem);

  void CovalentMinMaxDistance(BondedParticle * const Walker, BondedParticle * const OtherWalker, double &MinDistance, double &MaxDistance, bool IsAngstroem);

  MatrixContainer *BondLengthMatrix;
  double max_distance;
  bool IsAngstroem;
};

#endif /* BONDGRAPH_HPP_ */
